import subprocess
from pathlib import Path

from prompt_toolkit import print_formatted_text
from prompt_toolkit.application import create_app_session_from_tty
from prompt_toolkit.formatted_text import FormattedText
from prompt_toolkit.shortcuts import confirm


def print_info(info: str) -> None:
    print_formatted_text(FormattedText([("ansiblue", info)]))


def print_error(error: str) -> None:
    print_formatted_text(FormattedText([("ansired", error)]))


def replace_home_with_tilde(path: Path) -> Path:
    relative_path = path.relative_to(Path.home())
    return Path("~") / relative_path


supported_shells = ["zsh", "bash"]


def get_installed_shells() -> list[str]:
    shells = []
    for shell in supported_shells:
        if subprocess.run(["command", "-v", shell], stdout=subprocess.DEVNULL):
            shells.append(shell)
    return shells


def get_bundled_script_path(shell: str) -> Path:
    parent = Path(__file__).parent
    if shell == "zsh":
        return parent / "shelloracle.zsh"
    elif shell == "bash":
        return parent / "shelloracle.bash"


def get_script_path(shell: str) -> Path:
    if shell == "zsh":
        return Path.home() / ".shelloracle.zsh"
    elif shell == "bash":
        return Path.home() / ".shelloracle.bash"


def get_rc_path(shell: str) -> Path:
    if shell == "zsh":
        return Path.home() / ".zshrc"
    elif shell == "bash":
        return Path.home() / ".bashrc"


def write_script_home(shell: str) -> None:
    shelloracle = get_bundled_script_path(shell).read_bytes()
    destination = get_script_path(shell)
    destination.write_bytes(shelloracle)
    print_info(f"Successfully wrote key bindings to {replace_home_with_tilde(destination)}")


def update_rc(shell: str) -> None:
    rc_path = get_rc_path(shell)
    rc_path.touch(exist_ok=True)
    with rc_path.open("r") as file:
        zshrc = file.read()
    shelloracle_script = get_script_path(shell)
    line = f"[ -f {shelloracle_script} ] && source {shelloracle_script}"
    if line not in zshrc:
        with rc_path.open("a") as file:
            file.write("\n")
            file.write(line)
    print_info(f"Successfully updated {replace_home_with_tilde(rc_path)}")


def bootstrap() -> None:
    with create_app_session_from_tty():
        if not (shells := get_installed_shells()):
            print_error(f"No compatible shells found. Supported shells: {', '.join(supported_shells)}")
            return
        if confirm("Enable terminal keybindings and update rc ?", suffix=" ([y]/n) ") is False:
            return
        for shell in shells:
            write_script_home(shell)
            update_rc(shell)


if __name__ == '__main__':
    bootstrap()



import argparse

from . import shelloracle
from .bootstrap import bootstrap


def parse_args() -> argparse.Namespace:
    parser = argparse.ArgumentParser()
    parser.add_argument("--init", help="initialize shelloracle with scripts",
                        action="store_true")
    return parser.parse_args()


def main() -> None:
    args = parse_args()
    if args.init:
        bootstrap()
        exit(0)

    shelloracle.cli()


if __name__ == "__main__":
    main()



from __future__ import annotations

from collections.abc import MutableMapping, Iterator
from pathlib import Path
from typing import Any

import tomlkit

data_home = Path.home() / ".shelloracle"


def _default_config() -> tomlkit.TOMLDocument:
    config = tomlkit.document()
    shor_table = tomlkit.table()
    shor_table.add("provider", "Ollama")
    config.add("shelloracle", shor_table)
    return config


class Configuration(MutableMapping):
    filepath = data_home / "config.toml"

    def __init__(self) -> None:
        self._ensure_config_exists()

    def __getitem__(self, item: str) -> dict:
        with self.filepath.open("r") as file:
            config = tomlkit.load(file)
        return config[item]

    def __setitem__(self, key: str, value: Any) -> None:
        with self.filepath.open("r") as file:
            config = tomlkit.load(file)
        config[key] = value
        config.multiline = True
        with self.filepath.open("w") as file:
            tomlkit.dump(config, file)

    def __delitem__(self, key: str) -> None:
        with self.filepath.open("r") as file:
            config = tomlkit.load(file)
        del config[key]
        with self.filepath.open("w") as file:
            tomlkit.dump(config, file)

    def __iter__(self) -> Iterator[str]:
        with self.filepath.open("r") as file:
            config = tomlkit.load(file)
        return iter(config)

    def __len__(self) -> int:
        with self.filepath.open("r") as file:
            config = tomlkit.load(file)
        return len(config)

    def _ensure_config_exists(self) -> None:
        if self.filepath.exists():
            return
        data_home.mkdir(exist_ok=True)
        config = _default_config()
        with self.filepath.open("w") as file:
            tomlkit.dump(config, file)

    @property
    def provider(self) -> str:
        return self["shelloracle"]["provider"]


global_config = Configuration()



from abc import abstractmethod
from collections.abc import AsyncIterator
from typing import Protocol


class ProviderError(Exception):
    """LLM providers raise this error to gracefully indicate something has gone wrong."""


class Provider(Protocol):
    """
    LLM Provider Protocol

    All LLM backends must implement this interface.
    """
    name: str

    @abstractmethod
    def generate(self, prompt: str) -> AsyncIterator[str]:
        """
        This is an asynchronous generator method which defines the protocol that a provider implementation
        should adhere to. The method takes a prompt as an argument and produces an asynchronous stream
        of string results.

        :param prompt: A string value which serves as input to the provider's process of generating results.
        :return: An asynchronous generator yielding string results.
        """
        # If you are wondering why the 'generate' signature doesn't include 'async', see
        # https://mypy.readthedocs.io/en/stable/more_types.html#asynchronous-iterators


def get_provider(name: str) -> type[Provider]:
    """Imports and loads a requested provider

    :param name: the provider name
    :return: the requested provider
    """
    from .providers import Ollama, OpenAI
    providers = {
        Ollama.name: Ollama,
        OpenAI.name: OpenAI
    }
    return providers[name]



from __future__ import annotations

from typing import TypeVar, Generic, TYPE_CHECKING

from . import config

if TYPE_CHECKING:
    from ..provider import Provider

T = TypeVar("T")


class Setting(Generic[T]):
    def __init__(self, *, default: T | None = None) -> None:
        self.default = default

    def __set_name__(self, owner: type[Provider], name: str) -> None:
        self.name = name
        # Set the default value in the config dictionary if it doesn't exist
        provider_table = config.global_config.get("provider", {})
        provider_table.setdefault(owner.name, {}).setdefault(name, self.default)
        config.global_config["provider"] = provider_table

    def __get__(self, instance: Provider, owner: type[Provider]) -> T:
        return config.global_config.get("provider", {}).get(instance.name, {})[self.name]

    def __set__(self, instance: Provider, value: T) -> None:
        config.global_config.setdefault("provider", {}).setdefault(instance.name, {})[self.name] = value
